@@ -13,11 +13,11 @@
 # limitations under the License.
 
 import os
+import re
 import json
 import logging
 import datetime
 import xmlrpc.client
-from urllib.parse import urlparse
 
 from flask import abort
 from sqlalchemy.orm.attributes import flag_modified
@@ -25,7 +25,6 @@ from sqlalchemy.sql.expression import asc, desc, extract
 from sqlalchemy.orm import joinedload, aliased
 from sqlalchemy.sql import func, select
 import pytimeparse
-import clickhouse_driver
 import redis
 import boto3
 from azure.mgmt.subscription import SubscriptionClient
@@ -46,6 +45,7 @@ from . import minioops
 from . import schemaval
 from . import access
 from . import authn
+from . import chops
 
 
 log = logging.getLogger(__name__)
@@ -279,6 +279,32 @@ def get_branch_sequences(branch_id, token_info=None):
     return {'items': seqs, 'total': len(seqs)}, 200
 
 
+def update_branch_sequence(seq_id, body, token_info=None):
+    seq = BranchSequence.query.filter_by(id=seq_id).one_or_none()
+    if seq is None:
+        abort(404, "Branch sequence not found")
+
+    access.check(token_info, seq.branch.project_id, 'pwrusr',
+                 'only superadmin, project admin and project power user roles can get branch sequences')
+
+    value = body.get('value', None)
+    if value is None:
+        abort(400, "Missing value")
+
+    try:
+        value = int(value)
+    except Exception:
+        abort(400, "Incorrect value")
+
+    if value < -1:
+        abort(400, "Incorrect negative value")
+
+    seq.value = value
+    db.session.commit()
+
+    return seq.get_json(), 200
+
+
 def move_branch(branch_id, body, token_info=None):
     branch = Branch.query.filter_by(id=branch_id).one_or_none()
     if branch is None:
@@ -372,6 +398,11 @@ def create_secret(project_id, body, token_info=None):
     if project is None:
         abort(400, "Project with id %s does not exist" % project_id)
 
+    name = body['name']
+    m = re.search(r'[A-Za-z0-9_]+', name)
+    if not m:
+        abort(400, 'Secret name can have only A-Z, a-z, 0-9 or _ characters')
+
     if body['kind'] == 'ssh-key':
         kind = consts.SECRET_KIND_SSH_KEY
         data = dict(username=body['username'],
@@ -382,7 +413,7 @@ def create_secret(project_id, body, token_info=None):
     else:
         abort(400, "Wrong data")
 
-    secret = Secret(project=project, name=body['name'], kind=kind, data=data)
+    secret = Secret(project=project, name=name, kind=kind, data=data)
     db.session.commit()
 
     return secret.get_json(), 201
@@ -397,8 +428,13 @@ def update_secret(secret_id, body, token_info=None):
                  'only superadmin and project admin roles can modify a secret')
 
     if 'name' in body:
+        name = body['name']
+        m = re.search(r'^[A-Za-z0-9_]+$', name)
+        if not m:
+            abort(400, 'Secret name can have only A-Z, a-z, 0-9 or _ characters')
+
         old_name = secret.name
-        secret.name = body['name']
+        secret.name = name
         log.info('changed name from %s to %s', old_name, secret.name)
 
     if secret.kind == consts.SECRET_KIND_SIMPLE:
@@ -522,7 +558,14 @@ def update_stage(stage_id, body, token_info=None):
         if 'repo_branch' in body:
             stage.repo_branch = body['repo_branch']
         if 'repo_access_token' in body:
-            stage.repo_access_token = body['repo_access_token']
+            repo_access_token = body['repo_access_token']
+            secret = Secret.query.filter_by(project=stage.branch.project, name=repo_access_token).one_or_none()
+            if secret is None:
+                abort(400, "Secret '%s' for access token does not exist" % repo_access_token)
+            if secret.kind != consts.SECRET_KIND_SIMPLE:
+                abort(400, "Type of '%s' access token secret should be Simple Secret" % repo_access_token)
+            stage.repo_access_token = repo_access_token
+
         if 'schema_file' in body:
             stage.schema_file = body['schema_file']
         if 'repo_refresh_interval' in body:
@@ -959,6 +1002,14 @@ def update_settings(body, token_info=None):
                         continue
                     s.set_value(val)
 
+                    if name == 'clickhouse_log_ttl':
+                        ch = chops.get_clickhouse()
+                        # query = 'ALTER TABLE logs DELETE WHERE time < (now() - toIntervalMonth(%(months)s))'
+                        query = 'ALTER TABLE logs MODIFY TTL toDateTime(time) + toIntervalMonth(%(months)s)'
+                        params = {'months': int(val)}
+                        resp = ch.execute(query, params)
+                        log.info('set clickhouse_log_ttl to %s, resp: %s', val, resp)
+
     db.session.commit()
 
     settings = _get_settings()
@@ -981,7 +1032,7 @@ def get_diagnostics(token_info=None):
     }
 
     # check clickhouse
-    ch_url = os.environ.get('KRAKEN_CLICKHOUSE_URL', consts.DEFAULT_CLICKHOUSE_URL)
+    ch_url = chops.get_clickhouse_url()
     ch_open = srvcheck.is_service_open(ch_url)
     diags['clickhouse'] = {
         'name': 'ClickHouse',
@@ -1053,13 +1104,11 @@ def get_last_rq_jobs_names(token_info=None):
                  'only superadmin can get last rq job names')
 
     # get the last RQ jobs
-    ch_url = os.environ.get('KRAKEN_CLICKHOUSE_URL', consts.DEFAULT_CLICKHOUSE_URL)
-    o = urlparse(ch_url)
-    ch = clickhouse_driver.Client(host=o.hostname)
+    ch = chops.get_clickhouse()
 
     now = utils.utcnow()
     start_date = now - datetime.timedelta(hours=12111)
-    query = "select max(time) as mt, tool, count(*) from logs "
+    query = "select min(time) as mt, tool, count(*) from logs "
     query += "where service = 'rq' and tool != '' "
     query += "group by tool "
     query += "having mt > %(start_date)s "
@@ -1077,9 +1126,7 @@ def get_services_logs(services, level=None, token_info=None):
     access.check(token_info, '', 'admin',
                  'only superadmin can get services logs')
 
-    ch_url = os.environ.get('KRAKEN_CLICKHOUSE_URL', consts.DEFAULT_CLICKHOUSE_URL)
-    o = urlparse(ch_url)
-    ch = clickhouse_driver.Client(host=o.hostname)
+    ch = chops.get_clickhouse()
 
     query = "select time,message,service,host,level,tool from logs "
     where = []
@@ -1313,3 +1360,18 @@ def delete_tool(tool_id, token_info=None):
     db.session.commit()
 
     return {}, 200
+
+
+def update_branch_retention_policy(branch_id, body, token_info=None):
+    branch = Branch.query.filter_by(id=branch_id).one_or_none()
+    if branch is None:
+        abort(404, "Branch not found")
+
+    access.check(token_info, branch.project_id, 'pwrusr',
+                 'only superadmin, project admin and project power user roles can update branch retention policy')
+
+    branch.retention_policy = body
+    flag_modified(branch, 'retention_policy')
+    db.session.commit()
+
+    return branch.retention_policy, 200
