@@ -18,6 +18,7 @@ import logging
 import minio
 from minio.lifecycleconfig import LifecycleConfig, Expiration, Rule
 from minio.commonconfig import ENABLED, Filter
+from minio.error import S3Error
 import urllib3
 from urllib3.exceptions import MaxRetryError
 
@@ -68,8 +69,13 @@ def check_connection():
     mc = minio.Minio(minio_addr, access_key=root_user, secret_key=root_password, secure=False, http_client=http_client)
     try:
         mc.list_buckets()
-    except MaxRetryError:
+    except MaxRetryError as ex:
+        log.error('minio connection error: %s', ex)
         return False
+    except S3Error as ex:
+        log.error('minio connection error: %s', ex)
+        return False
+
     return True
 
 
