@@ -24,6 +24,8 @@ from flask import abort, Response
 import minio
 
 from .models import Run, Flow
+from . import users
+from . import access
 from . import consts
 
 
@@ -94,6 +96,13 @@ def serve_artifact(store_type, flow_id, run_id, path):
             abort(404, "Run not found")
         flow = run.flow
 
+
+    token_info = users.get_token_info_from_request()
+    if not token_info:
+        abort(401, "Missing Authorization header or kk_session_token cookie")
+    access.check(token_info, flow.branch.project_id, 'view',
+                 'only superadmin, project admin, project power user and project viewer roles can fetch artifacts')
+
     mt, _ = mimetypes.guess_type(path)
     if mt is None:
         mt = 'application/octet-stream'
